@@ -170,7 +170,7 @@ usage(FILE *out, const char *name)
           " -h, --help            display this help and exit.\n"
           " -v, --version         display version.\n"
           " -i, --ignorecase      match items case insensitively.\n"
-          " -F, --filter          filter entries for a given string.\n"
+          " -F, --filter          filter entries for a given string before showing the menu.\n"
           " -w, --wrap            wraps cursor selection.\n"
           " -l, --list            list items vertically with the given number of lines.\n"
           " -p, --prompt          defines the prompt text to be displayed.\n"
@@ -178,7 +178,15 @@ usage(FILE *out, const char *name)
           " -I, --index           select item at index automatically.\n"
           " -x, --password        hide input.\n"
           " -s, --no-spacing      disable the title spacing on entries.\n"
+          " -C, --no-cursor       ignore cursor events.\n"
+          " -T, --no-touch        ignore touch events.\n"
+          " -K, --no-keyboard     ignore keyboard events.\n"
+          " --binding             use alternative key bindings. Available options: vim\n"
+          " --fixed-height        prevent the display from changing height on filter.\n"
           " --scrollbar           display scrollbar. (none (default), always, autohide)\n"
+          " --counter             display a matched/total items counter. (none (default), always)\n"
+          " -e, --vim-esc-exits   exit bemenu when pressing escape in normal mode for vim bindings\n"
+          " --accept-single       immediately return if there is only one item.\n"
           " --ifne                only display menu if there are items.\n"
           " --fork                always fork. (bemenu-run)\n"
           " --no-exec             do not execute command. (bemenu-run)\n\n"
@@ -203,6 +211,7 @@ usage(FILE *out, const char *name)
           " -M, --margin          defines the empty space on either side of the menu. (wx)\n"
           " -W, --width-factor    defines the relative width factor of the menu (from 0 to 1). (wx)\n"
           " -B, --border          defines the width of the border in pixels around the menu. (wx)\n"
+          " -R  --border-radius   defines the radius of the border around the menu (0 = no curved borders).\n"
           " --ch                  defines the height of the cursor (0 = scales with line height). (wx)\n"
           " --cw                  defines the width of the cursor. (wx)\n"
           " --hp                  defines the horizontal padding for the entries in single line mode. (wx)\n"
@@ -264,7 +273,11 @@ do_getopt(struct client *client, int *argc, char **argv[])
         { "index",        required_argument, 0, 'I' },
         { "prefix",       required_argument, 0, 'P' },
         { "password",     no_argument,       0, 'x' },
+        { "fixed-height", no_argument,       0, 0x090 },
         { "scrollbar",    required_argument, 0, 0x100 },
+        { "counter",      required_argument, 0, 0x10a },
+        { "vim-esc-exits",no_argument,       0, 'e' },
+        { "accept-single",no_argument,       0, 0x11a },
         { "ifne",         no_argument,       0, 0x117 },
         { "fork",         no_argument,       0, 0x118 },
         { "no-exec",      no_argument,       0, 0x119 },
@@ -272,11 +285,15 @@ do_getopt(struct client *client, int *argc, char **argv[])
         { "grab",         no_argument,       0, 'f' },
         { "no-overlap",   no_argument,       0, 'n' },
         { "no-spacing",   no_argument,       0, 's' },
+        { "no-cursor",    no_argument,       0, 'C' },
+        { "no-touch",     no_argument,       0, 'T' },
+        { "no-keyboard",  no_argument,       0, 'K' },
         { "monitor",      required_argument, 0, 'm' },
         { "line-height",  required_argument, 0, 'H' },
         { "margin",       required_argument, 0, 'M' },
         { "width-factor", required_argument, 0, 'W' },
         { "border",       required_argument, 0, 'B' },
+        { "border-radius",required_argument, 0, 'R' },
         { "ch",           required_argument, 0, 0x120 },
         { "cw",           required_argument, 0, 0x125 },
         { "hp",           required_argument, 0, 0x122 },
@@ -300,6 +317,7 @@ do_getopt(struct client *client, int *argc, char **argv[])
         { "scb",          required_argument, 0, 0x114 },
         { "scf",          required_argument, 0, 0x115 },
         { "bdr",          required_argument, 0, 0x121 },
+        { "binding",      required_argument, 0, 0x128 },
 
         { "disco",       no_argument,       0, 0x116 },
         { 0, 0, 0, 0 }
@@ -309,13 +327,10 @@ do_getopt(struct client *client, int *argc, char **argv[])
      * Either break the interface and make them --sf, --sb (like they are now),
      * or parse them before running getopt.. */
 
-    /* Set default monitor index to -1 (maybe be overwritten with -m below). */
-    client->monitor = -1;
-
     for (optind = 0;;) {
         int32_t opt;
 
-        if ((opt = getopt_long(*argc, *argv, "hviwxcl:I:p:P:I:bfm:H:M:W:B:ns", opts, NULL)) < 0)
+        if ((opt = getopt_long(*argc, *argv, "hviwxcl:I:p:P:I:bfF:m:H:M:W:B:R:nsCTK", opts, NULL)) < 0)
             break;
 
         switch (opt) {
@@ -350,9 +365,21 @@ do_getopt(struct client *client, int *argc, char **argv[])
             case 'I':
                 client->selected = strtol(optarg, NULL, 10);
                 break;
+            case 0x090:
+                client->fixed_height = true;
+                break;
             case 0x100:
                 client->scrollbar = (!strcmp(optarg, "none") ? BM_SCROLLBAR_NONE : (!strcmp(optarg, "always") ? BM_SCROLLBAR_ALWAYS : (!strcmp(optarg, "autohide") ? BM_SCROLLBAR_AUTOHIDE : BM_SCROLLBAR_NONE)));
                 break;
+            case 0x10a:
+                client->counter = (!strcmp(optarg, "always"));
+                break;
+            case 'e':
+                client->vim_esc_exits = true;
+                break;
+            case 0x11a:
+                client->accept_single = true;
+                break;
             case 0x117:
                 client->ifne = true;
                 break;
@@ -381,7 +408,15 @@ do_getopt(struct client *client, int *argc, char **argv[])
             case 's':
                 client->no_spacing = true;
                 break;
-
+            case 'C':
+                client->no_cursor = true;
+                break;
+            case 'T':
+                client->no_touch = true;
+                break;
+            case 'K':
+                client->no_keyboard = true;
+                break;
             case 'H':
                 client->line_height = strtol(optarg, NULL, 10);
                 break;
@@ -394,6 +429,9 @@ do_getopt(struct client *client, int *argc, char **argv[])
             case 'B':
                 client->border_size = strtol(optarg, NULL, 10);
                 break;
+            case 'R':
+                client->border_radius = strtol(optarg, NULL, 10);
+                break;
             case 0x120:
                 client->cursor_height = strtol(optarg, NULL, 10);
                 break;
@@ -463,6 +501,13 @@ do_getopt(struct client *client, int *argc, char **argv[])
             case 0x121:
                 client->colors[BM_COLOR_BORDER] = optarg;
                 break;
+            case 0x128:
+                if(strcmp(optarg, "vim") == 0){
+                    client->key_binding = BM_KEY_BINDING_VIM;
+                } else {
+                    client->key_binding = BM_KEY_BINDING_DEFAULT;
+                }
+                break;
 
             case 0x116:
                 disco();
@@ -498,7 +543,7 @@ menu_with_options(struct client *client)
     if (!(menu = bm_menu_new(NULL)))
         return NULL;
 
-    client->fork = (client->force_fork || (bm_renderer_get_priorty(bm_menu_get_renderer(menu)) != BM_PRIO_TERMINAL)); 
+    client->fork = (client->force_fork || (bm_renderer_get_priorty(bm_menu_get_renderer(menu)) != BM_PRIO_TERMINAL));
 
     bm_menu_set_font(menu, client->font);
     bm_menu_set_line_height(menu, client->line_height);
@@ -512,12 +557,17 @@ menu_with_options(struct client *client)
     bm_menu_set_wrap(menu, client->wrap);
     bm_menu_set_monitor(menu, client->monitor);
     bm_menu_set_monitor_name(menu, client->monitor_name);
+    bm_menu_set_fixed_height(menu, client->fixed_height);
     bm_menu_set_scrollbar(menu, client->scrollbar);
+    bm_menu_set_counter(menu, client->counter);
+    bm_menu_set_vim_esc_exits(menu, client->vim_esc_exits);
     bm_menu_set_panel_overlap(menu, !client->no_overlap);
     bm_menu_set_spacing(menu, !client->no_spacing);
     bm_menu_set_password(menu, client->password);
     bm_menu_set_width(menu, client->hmargin_size, client->width_factor);
     bm_menu_set_border_size(menu, client->border_size);
+    bm_menu_set_border_radius(menu, client->border_radius);
+    bm_menu_set_key_binding(menu, client->key_binding);
 
     if (client->center) {
         bm_menu_set_align(menu, BM_ALIGN_CENTER);
@@ -546,20 +596,42 @@ run_menu(const struct client *client, struct bm_menu *menu, void (*item_cb)(cons
     bm_menu_set_highlighted_index(menu, client->selected);
     bm_menu_grab_keyboard(menu, true);
     bm_menu_set_filter(menu, client->initial_filter);
+    bm_menu_filter(menu);
 
-    if (client->ifne && !bm_menu_get_items(menu, NULL))
+    {
+    uint32_t item_count;
+    struct bm_item **items = bm_menu_get_filtered_items(menu, &item_count);
+
+    if (client->ifne && item_count == 0) {
         return BM_RUN_RESULT_CANCEL;
+    }
+
+    if (client->accept_single && item_count == 1) {
+        item_cb(client, *items);
+        return BM_RUN_RESULT_SELECTED;
+    }
+
+    }
 
     uint32_t unicode;
-    enum bm_key key;
-    struct bm_pointer pointer;
-    struct bm_touch touch;
+    enum bm_key key = BM_KEY_NONE;
+    struct bm_pointer pointer = {0};
+    struct bm_touch touch = {0};
     enum bm_run_result status = BM_RUN_RESULT_RUNNING;
     do {
-        bm_menu_render(menu);
-        key = bm_menu_poll_key(menu, &unicode);
-        pointer = bm_menu_poll_pointer(menu);
-        touch = bm_menu_poll_touch(menu);
+        if (!bm_menu_render(menu)) {
+            status = BM_RUN_RESULT_CANCEL;
+            break;
+        }
+        if (!client->no_keyboard) {
+            key = bm_menu_poll_key(menu, &unicode);
+        }
+        if (!client->no_cursor) {
+            pointer = bm_menu_poll_pointer(menu);
+        }
+        if (!client->no_touch) {
+            touch = bm_menu_poll_touch(menu);
+        }
     } while ((status = bm_menu_run_with_events(menu, key, pointer, touch, unicode)) == BM_RUN_RESULT_RUNNING);
 
     switch (status) {
