@@ -21,17 +21,26 @@ struct client {
     uint32_t monitor;
     uint32_t hmargin_size;
     uint32_t border_size;
+    uint32_t border_radius;
     float width_factor;
     bool bottom;
     bool center;
     bool grab;
     bool wrap;
+    bool fixed_height; 
+    bool counter;
+    bool vim_esc_exits; 
+    bool accept_single;
     bool ifne;
     bool no_overlap;
     bool no_spacing;
+    bool no_cursor;
+    bool no_touch;
+    bool no_keyboard;
     bool force_fork, fork;
     bool no_exec;
     bool password;
+    enum bm_key_binding key_binding;
     char *monitor_name;
 };
 
