@@ -103,7 +103,7 @@ struct render_api {
     /**
      * Tells underlying renderer to draw the menu.
      */
-    void (*render)(struct bm_menu *menu);
+    bool (*render)(struct bm_menu *menu);
 
     /**
      * Set vertical alignment of the bar.
@@ -352,11 +352,26 @@ struct bm_menu {
      */
     enum bm_filter_mode filter_mode;
 
+    /**
+     * Current fixed height mode.
+     */
+    bool fixed_height;
+
     /**
      * Current Scrollbar display mode.
      */
     enum bm_scrollbar_mode scrollbar;
 
+    /**
+     * Current vim escape exit mode.
+     */
+    bool vim_esc_exits; 
+
+    /**
+     * Current counter display mode.
+     */
+    bool counter;
+
     /**
      * Should selection be wrapped?
      */
@@ -382,6 +397,11 @@ struct bm_menu {
      */
     uint32_t border_size;
 
+    /**
+     * Border radius
+     */
+    uint32_t border_radius;
+
     /**
      * Is menu grabbed?
      */
@@ -411,6 +431,17 @@ struct bm_menu {
      * Is the menu needing a redraw ?
      */
     bool dirty;
+
+    /**
+     * Key binding that should be used.
+     */
+    enum bm_key_binding key_binding;
+
+    /**
+     * Vim binding specific variables.
+     */
+    char vim_mode;
+    uint32_t vim_last_key;
 };
 
 /* library.c */
