@@ -6,6 +6,8 @@
 #include <unistd.h>
 #include <assert.h>
 
+#include "vim.h"
+
 /**
  * Default font.
  */
@@ -53,6 +55,10 @@ bm_menu_new(const char *renderer)
 
     menu->dirty = true;
 
+    menu->key_binding = BM_KEY_BINDING_DEFAULT;
+    menu->vim_mode = 'i';
+    menu->vim_last_key = 0;
+
     uint32_t count;
     const struct bm_renderer **renderers = bm_get_renderers(&count);
 
@@ -354,6 +360,20 @@ bm_menu_get_border_size(struct bm_menu* menu)
     return menu->border_size;
 }
 
+void
+bm_menu_set_border_radius(struct bm_menu* menu, uint32_t border_radius)
+{
+    assert(menu);
+    menu->border_radius = border_radius;
+}
+
+uint32_t
+bm_menu_get_border_radius(struct bm_menu* menu)
+{
+    assert(menu);
+    return menu->border_radius;
+}
+
 uint32_t
 bm_menu_get_height(struct bm_menu *menu)
 {
@@ -414,6 +434,18 @@ char* bm_menu_get_color(const struct bm_menu *menu, enum bm_color color)
     return menu->colors[color].hex;
 }
 
+void
+bm_menu_set_fixed_height(struct bm_menu *menu, bool mode)
+{
+    menu->fixed_height = mode;
+}
+
+bool
+bm_menu_get_fixed_height(struct bm_menu *menu)
+{
+    return menu->fixed_height;
+}
+
 void
 bm_menu_set_scrollbar(struct bm_menu *menu, enum bm_scrollbar_mode mode)
 {
@@ -426,6 +458,30 @@ bm_menu_get_scrollbar(struct bm_menu *menu)
     return menu->scrollbar;
 }
 
+void
+bm_menu_set_vim_esc_exits(struct bm_menu *menu, bool mode)
+{
+    menu->vim_esc_exits = mode;
+}
+
+bool
+bm_menu_get_vim_esc_exits(struct bm_menu *menu)
+{
+    return menu->vim_esc_exits;
+}
+
+void
+bm_menu_set_counter(struct bm_menu *menu, bool mode)
+{
+    menu->counter = mode;
+}
+
+bool
+bm_menu_get_counter(struct bm_menu *menu)
+{
+    return menu->counter;
+}
+
 void
 bm_menu_set_align(struct bm_menu *menu, enum bm_align align)
 {
@@ -633,7 +689,7 @@ bm_menu_set_highlighted_index(struct bm_menu *menu, uint32_t index)
     bm_menu_get_filtered_items(menu, &count);
 
     if (count <= index)
-        return 0;
+        index = (count > 0 ? count - 1 : 0);
 
     if (menu->index != index)
         menu->dirty = true;
@@ -683,6 +739,11 @@ bm_menu_set_selected_items(struct bm_menu *menu, struct bm_item **items, uint32_
     return list_set_items_no_copy(&menu->selection, new_items, nmemb);
 }
 
+void
+bm_menu_set_key_binding(struct bm_menu *menu, enum bm_key_binding key_binding){
+    menu->key_binding = key_binding;
+}
+
 struct bm_item**
 bm_menu_get_selected_items(const struct bm_menu *menu, uint32_t *out_nmemb)
 {
@@ -723,13 +784,15 @@ bm_menu_get_filtered_items(const struct bm_menu *menu, uint32_t *out_nmemb)
     return list_get_items(&menu->items, out_nmemb);
 }
 
-void
+bool
 bm_menu_render(struct bm_menu *menu)
 {
     assert(menu);
 
     if (menu->renderer->api.render)
-        menu->renderer->api.render(menu);
+        return menu->renderer->api.render(menu);
+
+    return true;
 }
 
 void
@@ -751,7 +814,6 @@ bm_menu_filter(struct bm_menu *menu)
         size_t oldLen = strlen(menu->old_filter);
         addition = (oldLen < len && !memcmp(menu->old_filter, menu->filter, oldLen));
     }
-
     if (menu->old_filter && addition && menu->filtered.count <= 0)
         return;
 
@@ -852,13 +914,14 @@ menu_point_select(struct bm_menu *menu, uint32_t posx, uint32_t posy, uint32_t d
     }
     uint32_t selected_line = posy / line_height;
 
-    if (0 == selected_line) { // Mouse over title bar
-        return;
-    }
-
     assert(menu->lines != 0);
     uint16_t current_page_index = menu->index / menu->lines;
 
+    if (0 == selected_line) { // Mouse over title bar
+        bm_menu_set_highlighted_index(menu, current_page_index * menu->lines);
+        return;
+    }
+
     if (selected_line >= displayed) { // This might be useless
         return;
     }
@@ -937,6 +1000,20 @@ bm_menu_run_with_key(struct bm_menu *menu, enum bm_key key, uint32_t unicode)
     if (key != BM_KEY_NONE)
         menu->dirty = true;
 
+    if(menu->key_binding == BM_KEY_BINDING_VIM){
+        enum bm_vim_code code = bm_vim_key_press(menu, key, unicode, count, displayed);
+
+        switch(code){
+            case BM_VIM_CONSUME:
+                return BM_RUN_RESULT_RUNNING;
+            case BM_VIM_EXIT:
+                list_free_list(&menu->selection);
+                return BM_RUN_RESULT_CANCEL;
+            case BM_VIM_IGNORE:
+                break;
+        }
+    }
+
     switch (key) {
         case BM_KEY_LEFT:
             if (menu->filter) {
@@ -1170,7 +1247,7 @@ bm_menu_run_with_pointer(struct bm_menu *menu, struct bm_pointer pointer)
                 menu_prev(menu, count, menu->wrap);
             }
         }
-        if (pointer.event_mask & POINTER_EVENT_BUTTON && pointer.state == POINTER_STATE_PRESSED) {
+        if (pointer.event_mask & POINTER_EVENT_BUTTON && pointer.state == POINTER_STATE_RELEASED) {
             switch (pointer.button) {
                 case BM_POINTER_KEY_PRIMARY:
                     {
@@ -1192,11 +1269,11 @@ bm_menu_run_with_pointer(struct bm_menu *menu, struct bm_pointer pointer)
         }
     }
 
-    if (pointer.event_mask & (POINTER_EVENT_ENTER | POINTER_EVENT_MOTION)) {
+    if (pointer.event_mask & POINTER_EVENT_MOTION) {
         menu_point_select(menu, pointer.pos_x, pointer.pos_y, displayed);
     }
 
-    if (pointer.event_mask & POINTER_EVENT_BUTTON && pointer.state == POINTER_STATE_PRESSED) {
+    if (pointer.event_mask & POINTER_EVENT_BUTTON && pointer.state == POINTER_STATE_RELEASED) {
         switch (pointer.button) {
             case BM_POINTER_KEY_PRIMARY:
                 {
